ror_id2site = {
    "https://ror.org/010jaxs89": "EMBL Barcelona",
    "https://ror.org/02catss52": "EMBL-EBI",
    "https://ror.org/01zjc6908": "EMBL Grenoble",
    "https://ror.org/050589e39": "EMBL Hamburg",
    "https://ror.org/03mstc592": "EMBL Heidelberg",
    "https://ror.org/01yr73893": "EMBL Rome",
}
